/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qpainter.h>
#include <qcolor.h>
#include <qpoint.h>
#include <qpixmap.h>
#include <qrect.h>
#include <qfontmetrics.h>
#include <qfont.h>
#include <qsimplerichtext.h>
#include <qpalette.h>

#include <kdebug.h>
#include <kurl.h>

#include "../aux.h"

#include "../datastruct/DDataItemPix.h"
#include "DCanvasItemPix.h"

static const int maxcars = 15;
static const int maxlines  = 4;

static const int offset = 4;

	DCanvasItemPix::DCanvasItemPix(QWidget * parent, int idx) : 
DCanvasItemBase(parent, idx)
{
	m_type = DItem::Pix;
	updatePos();
	show();
}

DCanvasItemPix::~DCanvasItemPix()
{
	hide();
}

bool DCanvasItemPix::onWidget(const QPoint & p)
{
	return ( x() <= p.x() && x() + width() >= p.x() &&
			y() <= p.y() && y() + height() >= p.y() );
}

void DCanvasItemPix::drawContent(QPainter & p)
{
        if (! m_displayedpix.isNull())
        {
                p.drawPixmap( x(), y(), m_displayedpix, 0, 0, width(), height());
        }
        else
        {
                p.setBrush(data()->m_fillcolor);
                p.setPen(data()->m_outlinecolor);
                p.drawRoundRect( x() , y() , width(), height(), 0, 0 );
        }
}

void DCanvasItemPix::updatePos()
{
        QString displayedtext = QString("this text should not appear (bug)");


        DDataItemPix* t = static_cast<DDataItemPix*> (data());
        if (t)
                displayedtext = t->m_legend;

        // set size first

        QSimpleRichText rt(displayedtext, t->m_defaultFont);
        rt.adjustSize();

        DCanvasItemBase::setSize( rt.width() + 2*offset, t->m_pixtb.height() + rt.height() + 2*offset);
        m_displayedpix.resize( rt.width() + 2*offset, t->m_pixtb.height() + rt.height() + 2*offset);

        // set the painter

        QPainter p(&m_displayedpix);
        p.setFont( t->m_defaultFont );

        p.setBrush(data()->m_fillcolor);
        p.setPen(data()->m_outlinecolor);

        // paint

        p.drawRoundRect( 0 , 0 , m_displayedpix.width(), m_displayedpix.height(), 0, 0 );

	p.drawPixmap( QPoint((rt.width()-t->m_pixtb.width())/2 + offset, offset) , t->m_pixtb);
	
        QColorGroup cg;
        cg.setColor( QColorGroup::Background, data()->m_fillcolor );
        rt.draw(
                        &p,
                        offset,
                        offset + t->m_pixtb.height(),
                        QRect( 0, 0, width(), height() ),
                        cg);

        DCanvasItemBase::updatePos();
}
