/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qpoint.h>
#include <qpainter.h>
#include <qstring.h>
#include <qfont.h>
#include <qsimplerichtext.h>
#include <qpalette.h>

#include <kdebug.h>

#include "../aux.h"
#include "../datastruct/DDataItemRoot.h"
#include "DCanvasItemRoot.h"

static const int offset = 4;

DCanvasItemRoot::DCanvasItemRoot(QWidget * parent, int idx) : 
DCanvasItemBase(parent, idx)
{
	m_type = DItem::Root;
	updatePos();
	show();
}

DCanvasItemRoot::~DCanvasItemRoot()
{
	hide();
}

bool DCanvasItemRoot::onWidget(const QPoint & p)
{
	//TODO
        return ( x() <= p.x() && x() + width() >= p.x() &&
                       y() <= p.y() && y() + height() >= p.y() );
}

void DCanvasItemRoot::drawContent(QPainter & p)
{
	if (! m_displayedpix.isNull())
	{
		p.drawPixmap( x(), y(), m_displayedpix, 0, 0, width(), height());
	}
	else
	{
		p.setBrush(data()->m_fillcolor);
		p.setPen(data()->m_outlinecolor);
		p.drawRoundRect( x() , y() , width(), height(), 0, 0 );
	}
}

void DCanvasItemRoot::updatePos()
{
	QString displayedtext = QString("this text should not appear (bug)");


	DDataItemRoot* t = static_cast<DDataItemRoot*> (data());
	if (t)
		displayedtext = t->m_title;

	// set size first

	QSimpleRichText rt(displayedtext, t->m_defaultFont);
	rt.adjustSize();

	DCanvasItemBase::setSize( rt.width() + offset*2, rt.height() + offset*2);
	m_displayedpix.resize( rt.width() + offset*2, rt.height() + offset*2);

	// set the painter
	
	QPainter p(&m_displayedpix);
	p.setFont( t->m_defaultFont );

	p.setBrush(data()->m_fillcolor);
	p.setPen(data()->m_outlinecolor);

	// paint
	
	p.drawRoundRect( 0 , 0 , m_displayedpix.width(), m_displayedpix.height(), 4, 4 );

        QColorGroup cg;
        cg.setColor( QColorGroup::Background, data()->m_fillcolor );
        rt.draw(
                        &p,
                        offset,
                        offset,
                        QRect( 0, 0, width(), height() ),
                        cg);

        DCanvasItemBase::updatePos();

}

