/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kiconloader.h>
#include <klocale.h>

#include "settings.h"
#include "DCanvasView.h"
#include "DCanvasPopup.h"


DCanvasPopup::DCanvasPopup(QWidget *parent) : KPopupMenu(parent)
{
	canvas = (DCanvasView*) parent;

	//////////////////////////////
	//////////////////////////////
	// new object on the canvas / canvas manipulation
	// 

	KPopupMenu *new_pop = new KPopupMenu(this, i18n("New"));
	new_pop->insertItem(SmallIcon("source"), i18n("Root"), e_root);
	new_pop->insertItem(SmallIcon("source"), i18n("Text"), e_text);
	new_pop->insertItem(SmallIcon("source"), i18n("Picture"), e_pix);

	insertItem(SmallIcon("filenew"), i18n("New"),new_pop);
	insertSeparator();

	if ( canvas->hasRoot() && ! Settings::multipleRoot() )
		setItemEnabled(e_root, false);

	setItemEnabled( e_text, canvas->hasRoot() );
	setItemEnabled( e_pix,  canvas->hasRoot() );


	if (new_pop)
		connect(new_pop, SIGNAL(activated(int)), this, SIGNAL(activated(int)));

	///////////////////////////////
	///////////////////////////////
	// edit actions
	//

	insertItem(i18n("Cut"), e_cut);
	insertItem(i18n("Copy"), e_copy);
	insertItem(i18n("Paste"), e_paste);

	setItemEnabled(e_cut, false);
	setItemEnabled(e_copy, false);
	setItemEnabled(e_paste, false);

	if ( canvas->itemsAreSelected() )
	{
		insertSeparator();
		insertItem(i18n("Delete"), e_delete);
	}


	insertSeparator();

	////////////////////////////////
	////////////////////////////////
	// disconnect actions
	//

	KPopupMenu *disc_pop = new KPopupMenu(this, i18n("Disconnect"));
	// disc_pop ~ disconnect popup :)

	disc_pop->insertItem(i18n("Object(s)"), e_discon_obj);
	disc_pop->insertItem(i18n("Tree"), e_discon_tree);
	disc_pop->insertItem(i18n("Subtree"), e_discon_subtree);

	insertItem( i18n("Disconnect"), disc_pop);

	if (disc_pop)
		connect(disc_pop, SIGNAL(activated(int)), this, SIGNAL(activated(int)));

	setItemEnabled(e_discon_subtree, canvas->oneItemIsSelected() );
	setItemEnabled(e_discon_tree, canvas->isRootSelected() );

	disc_pop->setEnabled( canvas->itemsAreSelected() );

	insertSeparator();


	///////////////////////////////
	///////////////////////////////
	// TODO : reorganize the tree automatically ?
	//

	insertItem("Unclutter Tree", e_unclutter);
	setItemEnabled(e_unclutter, false);
	insertSeparator();

	///////////////////////////////
	///////////////////////////////
	// properties actions
	//

	//insertItem(i18n("Colors"), e_color);
	//setItemEnabled(e_color, canvas->itemsAreSelected() );

	KPopupMenu *col_pop = new KPopupMenu(this, i18n("Colors"));

	col_pop->insertItem(i18n("Original color"), Settings::EnumColorMode::default_);
	col_pop->insertItem(i18n("Clear"), Settings::EnumColorMode::clear_);
	col_pop->insertItem(i18n("Important"), Settings::EnumColorMode::important_);
	col_pop->insertItem(i18n("Opposition"), Settings::EnumColorMode::opposition_);
	col_pop->insertItem(i18n("Enumeration"), Settings::EnumColorMode::enumeration_);
	col_pop->insertItem(i18n("Todo"), Settings::EnumColorMode::todo_);
	col_pop->insertSeparator();
	col_pop->insertItem(i18n("Custom..."), Settings::EnumColorMode::custom_);

	connect(col_pop, SIGNAL(activated(int)), this, SIGNAL(activated(int)));

	insertItem( i18n("Colors"), col_pop);		
	col_pop->setEnabled( canvas->itemsAreSelected() );
	//setItemEnabled(col_pop, canvas->itemsAreSelected() );

	///////////////////

	insertItem(i18n("Font properties"), e_font);
	setItemEnabled(e_font, canvas->itemsAreSelected() );

	insertItem(i18n("Object properties"), e_properties);
	setItemEnabled(e_properties, canvas->oneItemIsSelected() );

}




