/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qpoint.h>
#include <qobjcoll.h>
#include <qevent.h>
#include <qpen.h>
#include <qcursor.h>
#include <qfont.h>
#include <qpixmap.h>

#include <kdebug.h>
#include <kfontdialog.h>
#include <kcolordialog.h>

#include "DCanvasItemBase.h"
#include "DCanvasPopup.h"
#include "DCanvasView.h"

#include "../gui/settings.h"
#include "../datastruct/constants.h"
#include "../datastruct/DDataItemBase.h"
#include "../datastruct/DDataControl.h"


#include "DCanvasItemRoot.h"
#include "DCanvasItemText.h"
#include "DCanvasItemPix.h"

#include "DCanvasPos.h"


static const int s_linez = 50;



DCanvasView::DCanvasView(QWidget* parent, const char*) : QCanvasView(parent), DGuiView(parent) 
{
	m_pressed = false;
	m_multiselect = false;
	m_menu = NULL;
	m_data = NULL;
	m_rootItem = NULL;
	m_rubberLine = NULL;
	m_startItem = NULL;
	m_currentAction = DCanvasView::act_point;
	setCursor(ArrowCursor);

	m_itemtosort = NOITEM;
	m_sortcount = 0;

	setCanvas(new QCanvas());
	canvas()->setAdvancePeriod(40);

	settingsChanged();
}

DCanvasView::~DCanvasView()
{

}

DCanvasItemBase* DCanvasView::canvasItem( int id )
{
	if (id == NOITEM)
		return NULL;

	return (DCanvasItemBase*) Item( id );
}

void DCanvasView::contentsMousePressEvent(QMouseEvent* me)
{
	// kdWarning()<<"DCanvasView::contentsMousePressEvent"<<endl;
	// remove context menu

	if (m_menu != NULL)
	{
		m_menu->hide();
		disconnect(m_menu, SIGNAL(activated(int)), this, SLOT(popupMenuSel(int)));
		delete m_menu;
		m_menu = NULL;
	}


	if (me->button() == LeftButton)
		m_pressed = true;
	m_moveStart = me->pos();

	int newitem = locateItem( me->pos() );

	if (me->button() == LeftButton)
	{
		if (act_point == m_currentAction)
		{
			if ( ! m_multiselect)
				deselectAll();


			if ( newitem != NOITEM )
			{
				canvasItem(newitem)->setOffset( me->pos() );
				if (m_selectedList.contains(newitem))
				{
					canvasItem(newitem)->setSelected(false);
					m_selectedList.remove(newitem);
				}
				else
				{
					addItemToSelection( newitem );
				}

				// use this if you want to debug the structure of the tree
				//dataTree()->debugItem(newitem);
			}

			if ( m_multiselect )
			{
				for ( m_it = m_selectedList.begin(); m_it != m_selectedList.end(); ++m_it )
				{
					canvasItem(*m_it)->setOffset( me->pos() );
				}
			}
		}

		if (act_link == m_currentAction)
		{
			if (m_startItem)
			{
				m_startItem->setSelected(false);
			}

			m_startItem = canvasItem( locateItem(me->pos()) );

			if (m_startItem)
			{	
				m_startItem->setSelected(true);
			}
		}

		if (act_sort == m_currentAction)
		{

			int child = locateItemPos(me->pos());

			if (child != NOITEM)
			{
				// set the position of a child
				// kdWarning()<<"sort child on mouse press "<<child<<endl;
				dataItem(m_itemtosort)->setChildIdx(child, m_sortcount);

				m_sortcount++;
				if (m_sortcount > dataItem(m_itemtosort)->countChildren())
					m_sortcount = 0;

				updateItem(m_itemtosort);
				dataTree()->notifyChildren(m_itemtosort);
			}
			else
			{
				// select another parent for sorting his children
				m_sortcount = 0;
				if (m_itemtosort != NOITEM)
				{
					canvasItem(m_itemtosort)->setSelected(false);
					for (int i=0; i<dataItem(m_itemtosort)->countChildren(); i++)
					{
						canvasItem(dataItem(m_itemtosort)->childNum(i))->setDisplayIdx(false);
					}
				}

				m_itemtosort = newitem;

				if (m_itemtosort != NOITEM)
				{
					canvasItem(m_itemtosort)->setSelected(true);
					for (int i=0; i<dataItem(m_itemtosort)->countChildren(); i++)
					{
						canvasItem(dataItem(m_itemtosort)->childNum(i))->setDisplayIdx(true);
					}
				}
			}

			canvas()->update();
		}
	}


	if (me->button() == RightButton)
	{
		if ( ! m_multiselect && !m_selectedList.contains(newitem) )
			deselectAll();

		if ( newitem != NOITEM)
		{
			canvasItem(newitem)->setOffset( me->pos() );
			if (!m_selectedList.contains(newitem))
				addItemToSelection( newitem );
		}
		else
		{
			deselectAll();
		}

		m_menu = new DCanvasPopup(this);
		m_menu->popup(me->globalPos());
		connect(m_menu, SIGNAL(activated(int)), this, SLOT(popupMenuSel(int)));
	}
}

void DCanvasView::addItemToSelection( int id )
{
	if (id == NOITEM)
		return;

	canvasItem(id)->setSelected(true);

	if (m_selectedList.contains(id) == 0)
		m_selectedList.push_back(id);
}


void DCanvasView::contentsMouseDoubleClickEvent(QMouseEvent* me)
{
	int newitem = locateItem( me->pos() );
	if ( newitem != NOITEM )
	{
		dataTree()->configure( newitem );
	}
	deselectAll();
}

void DCanvasView::contentsMouseMoveEvent(QMouseEvent* me)
{
	if (m_pressed && m_currentAction == act_point)
	{
		for ( m_it = m_selectedList.begin(); m_it != m_selectedList.end(); ++m_it )
		{
			canvasItem(*m_it)->mouseMoveEvent(me);

			// update the children threads
			dataTree()->notifyChildren(*m_it);
		}
	}

	if (m_currentAction == act_link)
	{
		if (m_rubberLine)
		{
			m_rubberLine->setPoints(m_moveStart.x(), m_moveStart.y(), me->pos().x(), me->pos().y());
		}
	}
}

void DCanvasView::contentsMouseReleaseEvent(QMouseEvent* me)
{
	m_pressed = false;

	if (m_currentAction == act_link)
	{
		if (m_rubberLine)
		{
			m_rubberLine->setPoints(0, 0, 0, 0);
		}

		if (m_startItem)
		{
			m_startItem->setSelected(false);
			int id = locateItem(me->pos());

			if (id != NOITEM)
			{
				dataTree()->linkItems( m_startItem->Id(), id );
				updateItem(id);
			}
			updateItem(m_startItem->Id());
		}
		m_startItem = NULL;
	}
}

int DCanvasView::locateItem( const QPoint& p)
{
	DCanvasItemBase* item = NULL;
	DCanvasItemBase* obj  = NULL;
	QCanvasItemList l = canvas()->collisions(p);
	QCanvasItemList::iterator it;

	double z=-1;

	for ( it = l.begin(); it != l.end(); ++it  )
	{
		if ( (*it)->rtti() == QCanvasItem::Rtti_Line || (*it)->rtti() == QCanvasItem::Rtti_Ellipse)
			continue;

		obj = (DCanvasItemBase*) (*it);

		if ( obj->z() > z )
		{
			z = obj->z();
			item = obj;
		}
	}

	if (!item)
		return NOITEM;

	return item->Id();
}

int DCanvasView::locateItemPos( const QPoint& p)
{
	DCanvasPos* pos = NULL;
	DCanvasPos* obj  = NULL;
	QCanvasItemList l = canvas()->collisions(p);
	QCanvasItemList::iterator it;

	for ( it = l.begin(); it != l.end(); ++it  )
	{
		if ( (*it)->rtti() != QCanvasItem::Rtti_Ellipse)
			continue;

		obj = (DCanvasPos*) (*it);
		break;
	}

	pos = obj;

	if (!pos)
		return NOITEM;

	return pos->Id();
}

void DCanvasView::keyPressEvent( QKeyEvent * e )
{
	// kdWarning()<<"key pressed"<<endl;
	switch (e->key())
	{
		case Qt::Key_Shift:
			m_multiselect = true;
			// kdWarning()<<"shift key pressed"<<endl;
			break;

		case Qt::Key_Delete:
		case Qt::Key_Backspace:
			removeSelectedItems();
			break;
		default:
			break;
	}
}

void DCanvasView::keyReleaseEvent( QKeyEvent * e )
{
	// kdWarning()<<"key released"<<endl;
	if (e->key() == Qt::Key_Shift )
	{
		m_multiselect = false;
		// kdWarning()<<"shift key released"<<endl;
	}
}

void DCanvasView::deselectAll()
{
	for ( m_it = m_selectedList.begin(); m_it != m_selectedList.end(); ++m_it )
	{
		canvasItem(*m_it)->setSelected(false);
	}
	m_selectedList.clear();
}

bool DCanvasView::oneItemIsSelected() const
{
	if (m_selectedList.count() == 1)
		return true;		
	return false;
}

bool DCanvasView::itemsAreSelected() const
{
	return (m_selectedList.count() > 0);
}

bool DCanvasView::isRootSelected() const
{
	return false;
}

bool DCanvasView::erasable()
{
	// TODO
	/*	if (! m_selected )
		return 0;

		if ( m_selected == m_rootItem )
		return 0;
		*/
	return 1;
}

void DCanvasView::popupMenuSel(int sel)
{
	switch (sel)
	{
		case DCanvasPopup::e_root:
			if (hasRoot() && ! Settings::multipleRoot())
			{
				// kdWarning()<<"multiple root are not supposed to be allowed"<<endl;
				break;
			}
			requestItem(DItem::Root);
			break;
		case DCanvasPopup::e_text:
			requestItem(DItem::Text);
			break;
		case DCanvasPopup::e_pix:
			requestItem(DItem::Pix);
			break;

		case DCanvasPopup::e_delete:
			removeSelectedItems();
			break;

		case DCanvasPopup::e_discon_obj:
			for ( m_it = m_selectedList.begin(); m_it != m_selectedList.end(); ++m_it )
				dataTree()->disconnectItem( *m_it );
			break;

		case DCanvasPopup::e_discon_subtree:
			if (oneItemIsSelected())
				dataTree()->killFamily( m_selectedList[0] );
			break;

		case DCanvasPopup::e_font:
			{
				QFont font;

				if (oneItemIsSelected())
					font = dataItem(m_selectedList[0])->m_defaultFont;
				else
					font = Settings::canvasFont();

				int result = KFontDialog::getFont( font );
				if ( result == KFontDialog::Accepted )
				{
					for ( m_it = m_selectedList.begin(); m_it != m_selectedList.end(); ++m_it )
					{
						dataItem(*m_it)->m_defaultFont = font;
						canvasItem(*m_it)->updatePos();
					}
				}
			}
			break;

		case DCanvasPopup::e_properties:
			if (oneItemIsSelected())
				dataTree()->configure( m_selectedList[0] );
			break;

		case Settings::EnumColorMode::custom_:
			{
				QColor myColor;
				int result = KColorDialog::getColor( myColor );

				if ( result == KColorDialog::Accepted )
					applyQuickColor( myColor );
			}
			break;

		case Settings::EnumColorMode::default_:
			for ( m_it = m_selectedList.begin(); m_it != m_selectedList.end(); ++m_it )
			{
				dataItem(*m_it)->initColor();
				canvasItem(*m_it)->updatePos();
			}
			break;

		case Settings::EnumColorMode::clear_:
		case Settings::EnumColorMode::important_:
		case Settings::EnumColorMode::opposition_:
		case Settings::EnumColorMode::enumeration_:
		case Settings::EnumColorMode::todo_:
			applyQuickColor( Settings::fillcolor(sel) );
			break;
		default:
			break;
	}

	// no multiselect
	m_multiselect = false;
}

void DCanvasView::applyQuickColor( const QColor & color )
{
	for ( m_it = m_selectedList.begin(); m_it != m_selectedList.end(); ++m_it )
	{
		dataItem(*m_it)->m_fillcolor = color;
		canvasItem(*m_it)->updatePos();
	}
}

void DCanvasView::settingsChanged()
{
	if (!canvas())
		return;

	if (Settings::canvasScrolling())
	{
		canvas()->resize(2000, 2000);
		setHScrollBarMode(QScrollView::Auto);
		setVScrollBarMode(QScrollView::Auto);
	}
	else
	{
		canvas()->resize(width(), height());
		setHScrollBarMode(QScrollView::AlwaysOff);
		setVScrollBarMode(QScrollView::AlwaysOff);
	}

	canvas()->setBackgroundColor(Settings::col_background());
}

void DCanvasView::removeSelectedItems()
{
	while ( m_selectedList.count() > 0 )
	{
		dataTree()->removeItem( m_selectedList.front() );
	}
}

void DCanvasView::createItem( int idx )
{
	if (!m_data)
		return;

	DCanvasItemBase* item = NULL;
	DItem::itype t = dataTree()->ItemType(idx);

	switch (t)
	{
		case DDataItemBase::Text:
			item = new DCanvasItemText(this, idx);
			break;
		case DDataItemBase::Pix:
			item = new DCanvasItemPix(this, idx);
			break;
		case DDataItemBase::Root:
			item = new DCanvasItemRoot(this, idx);
			break;
		default:
			// kdWarning()<<"DCanvasView::newitem : tried to create invalid item"<<endl;
			break;
	}

	if (!item)
		return;

	registerItem(item);
	if (dataItem(idx)->x() == 0 && dataItem(idx)->y() == 0)
		item->move( (double) m_moveStart.x(), (double) m_moveStart.y());
	//else
	// kdWarning()<<"x and y are :"<<dataItem(idx)->x()<<" "<<dataItem(idx)->y()<<endl;

	item->show();
}

bool DCanvasView::hasRoot()
{
	if (! dataTree())
		return false;

	if ( dataTree()->rootID() != NOITEM)
		return true;

	return false;
}

void DCanvasView::resizeEvent(QResizeEvent *e)
{
	if (! Settings::canvasScrolling())
	{
		canvas()->resize( e->size().width(), e->size().height() );
	}
	else
	{
		QCanvasView::resizeEvent(e);
	}
}

void DCanvasView::unregisterItem(int id)
{
	DCanvasItemBase* item = NULL;
	if (isRegistered(id))
	{
		item = (DCanvasItemBase*) Item(id);
		item->hide();
		m_map.remove(id);
	}

	// auto-delete
	delete item;
}

void DCanvasView::setMode(ModeActions action)
{
	//TODO
	// kdWarning()<<"DCanvasView::setMode sets action : "<<action<<endl;
	deselectAll();
	m_currentAction = action;

	switch (m_currentAction)
	{
		case act_point:
			setCursor(ArrowCursor);
			break;
		case act_link:
			setCursor(CrossCursor);
			break;
		case act_sort:
			setCursor(PointingHandCursor);
			break;
		default:
			break;
	}

	if (m_currentAction == act_link)
	{
		m_rubberLine = new QCanvasLine(canvas());
		m_rubberLine->show();
		m_rubberLine->setPoints(0,0, 0,0);
		m_rubberLine->setPen( QColor(Qt::black) );
		m_rubberLine->setZ( s_linez );
	}
	else
	{
		if (m_rubberLine)
		{
			m_rubberLine->hide();
			delete m_rubberLine;
			m_rubberLine = NULL;
		}
	}

	if (m_currentAction != act_sort)
	{
		if (m_itemtosort != NOITEM)
		{
			canvasItem(m_itemtosort)->setSelected(false);
			for (int i=0; i<dataItem(m_itemtosort)->countChildren(); i++)
			{
				canvasItem(dataItem(m_itemtosort)->childNum(i))->setDisplayIdx(false);
			}
		}		      
	}
}


void DCanvasView::updateItem(int id)
{
	DCanvasItemBase* item = canvasItem(id);
	if (item)
	{
		//// kdWarning()<<"DCanvasView::updateItem "<<item<<endl;
		item->updatePos();
		item->updateLink();
	}
}

void DCanvasView::plug()
{
	// kdWarning()<<"DCanvasView::plug"<<endl;
	if (!dataTree())
		return;
	connect(dataTree(), SIGNAL( itemChanged(int) ), this, SLOT( updateItem(int) ) );
	connect(dataTree(), SIGNAL( itemCreated(int) ), this, SLOT( createItem(int) ) );
	connect(dataTree(), SIGNAL( itemRemoved(int) ), this, SLOT( removeItem(int) ) );
}

void DCanvasView::unplug()
{
	// kdWarning()<<"DCanvasView::unplug"<<endl;
	if (!dataTree())
		return;
	disconnect(dataTree(), SIGNAL( itemChanged(int) ), this, SLOT( updateItem(int) ) );
	disconnect(dataTree(), SIGNAL( itemCreated(int) ), this, SLOT( createItem(int) ) );
	disconnect(dataTree(), SIGNAL( itemRemoved(int) ), this, SLOT( removeItem(int) ) );
}

void DCanvasView::removeItem(int id)
{
	unregisterItem(id);
	m_selectedList.remove(id);
	if (m_itemtosort == id)
		m_itemtosort = NOITEM;

	canvas()->update();
}

#include "DCanvasView.moc"

