/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _DCANVASVIEW_H
#define _DCANVASVIEW_H

#include <qcanvas.h>
#include "../gui/DGuiView.h"
#include "../gui/DGuiItem.h"
#include "../datastruct/DItem.h"

#include <qcanvas.h>
#include <qvaluelist.h>


class QPoint;
class DCanvasItemBase;
class DCanvasPopup;
class QResizeEvent;

class DCanvasView : public QCanvasView, public DGuiView
{
	Q_OBJECT

	public:
		enum ModeActions {
			act_point,
			act_link,
			act_sort
		};

		DCanvasView( QWidget* parent=0, const char* name=0 );
		~DCanvasView();

		void contentsMouseReleaseEvent(QMouseEvent* me);
		void contentsMouseMoveEvent(QMouseEvent* me);
		void contentsMouseDoubleClickEvent(QMouseEvent* me);
		void contentsMousePressEvent(QMouseEvent* me);

		// TODO
		bool isRootSelected() const;
		bool oneItemIsSelected() const;
		bool itemsAreSelected() const;

		//dissertWidget * root();

		void deselectAll();

		// TODO
		//itemWidget* addConcept(dissertWidget *parentitem, const QString &text);
		//dissertWidget* setTheme(const QString &text);

		// void init();

		bool erasable();
		void setMode(ModeActions action);


		DCanvasItemBase* canvasItem(int);


		bool hasRoot();
		
		
	protected:
		void resizeEvent( QResizeEvent* );
		void keyPressEvent( QKeyEvent* );
		void keyReleaseEvent( QKeyEvent* );
		
		void unregisterItem(int id);


	private:
		//TODO
		DCanvasItemBase* m_rootItem;


		// start item
		DCanvasItemBase* m_startItem;
		
		// rubber band for link mode
		QCanvasLine* m_rubberLine;
		
		DCanvasPopup* m_menu;
		//rootWidget *m_rootwidget;
		QPoint m_moveStart;

		void plug();
		void unplug();
		void applyQuickColor(const QColor &);
	
		int m_itemtosort;
		int m_sortcount;
		
		ModeActions m_currentAction;
		//void linkIdeas(dissertWidget *n, dissertWidget *p);
		//
	
		void addItemToSelection( int );
		void removeSelectedItems();

	

		
		// on mouse press, locate the corresponding item
		int locateItem( const QPoint& );

		// same as above, but used to change the position of an item
		int locateItemPos( const QPoint& );

		// is the mouse left button currently pressed ?
		bool m_pressed;
		
		// multiselect : is the shift key pressed ? - want to select more than one item
		bool m_multiselect;

		// selected items - by id
		QValueList<int> m_selectedList;
		QValueList<int>::iterator m_it;

	public slots:
		void popupMenuSel(int sel);
		void settingsChanged();
		void updateItem(int id);
		void createItem(int);
		void removeItem(int);
	
};



#endif // _DCANVASVIEW_H

