/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _DDATACONTROL_H
#define _DDATACONTROL_H

#include <qdatastream.h>
#include <qmap.h>
#include <qobject.h>
#include "DBase.h"
#include "DItem.h"

class DissertParser;
class DGenerator;
class QTextStream;
class KURL;
class DDataItemBase;
class DDataControl : public QObject, public DBase
{
	Q_OBJECT

	friend class DissertParser;
	public:
		DDataControl(QObject * parent = 0, const char * name = 0);
		~DDataControl();

		void createItem( DItem::itype );
		void removeItem( int );
		
		void linkItems( int , int );
		void unlinkItems( int , int );

		void setOrphan(int);
		void killChildren(int);
		void killFamily(int);

		void configure(int);
		void disconnectItem(int);

		void notifyChildren(int id);
		bool saveToFile(const KURL &);
		bool loadFromFile(const KURL &);

		void printTree(QTextStream&);

		int countItems();
		int idNum(int);
		int rootID();

		bool canGenerate();


		void debugItem(int id);
		void checkConsistency(); // makes the tree a real tree
		
	signals:
		void itemChanged(int);
		void itemCreated(int);
		void itemRemoved(int);

	protected:
		DDataItemBase* createItem(DItem::itype type, int id);
		
	public slots:
		void clearDocument();			
		
	private:
		int m_lastid;
		DDataItemBase* DDataControl::dataItem(int id);
};

#endif // _DDATACONTROL_H

