/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qtextstream.h>
#include <qcolor.h>
#include <qfont.h>

#include <kdebug.h>

#include "../gui/settings.h"
#include "DDataItemBase.h"


DDataItemBase::DDataItemBase(int id) : DItem(id)
{
	m_parent = -1;
	m_framecolor.setRgb(250,0,0);
	m_conncolor.setRgb(250,0,250);

	m_defaultFont = Settings::canvasFont();

	m_x = 0;
	m_y = 0;
}

DDataItemBase::~DDataItemBase()
{
	m_children.clear();
}

int DDataItemBase::Parent()
{
	return m_parent;
}

void DDataItemBase::addChild(int id)
{
	if ( id == Id() )
		return;

	m_children.push_back(id);
}

void DDataItemBase::setParent(int id)
{
	if ( id == Id() )
		return;

	m_parent = id;
}

void DDataItemBase::setNoParent()
{
	setParent(-1);
}

void DDataItemBase::removeChild(int id)
{
	if ( id == Id() )
		return;

	m_children.remove( id );
}

int DDataItemBase::countChildren() const
{
	return m_children.count();
}

int DDataItemBase::childNum(int idx) const
{
	if (idx >= 0 && idx < countChildren())
		return m_children[idx]; 
	else
		return -1;
}

int DDataItemBase::childIdx(int id) const
{
	return m_children.findIndex(id);
}

void DDataItemBase::setChildIdx( int id, int pos)
{
	if (pos <0 || pos >= countChildren())
		return;

	if (! m_children.contains(id) )
		return;

	int intruderid = m_children[pos];
	int newidxforintruder = m_children.findIndex(id);

	m_children[newidxforintruder] = intruderid;
	m_children[pos]               = id;
}

void DDataItemBase::setXY(int x, int y)
{
	m_x = x;
	m_y = y;
}

void DDataItemBase::printSelf(QTextStream& s)
{
	s<<"<item>\n";
	printData(s);
	s<<"</item>\n";
}

void DDataItemBase::printData(QTextStream& s)
{
	// id and type
	s<<" <id>"<<Id()<<"</id>\n";
	s<<" <type>"<<TypeName(Type())<<"</type>\n";

	// position on canvas
	s<<" <xpos>"<<m_x<<"</xpos>"<<"<ypos>"<<m_y<<"</ypos>\n";
	
	// features
	s<<" <defaultfont>"<<m_defaultFont.toString()<<"</defaultfont>\n";
	s<<" <fillcolor>"<<m_fillcolor.name()<<"</fillcolor>\n";
	s<<" <outlinecolor>"<<m_outlinecolor.name()<<"</outlinecolor>\n";
	s<<" <textcolor>"<<m_textcolor.name()<<"</textcolor>\n";
	 
	// family
	s<<" <parent>"<<Parent()<<"</parent>\n";
	for (unsigned int i = 0; i <m_children.count(); i++)
	{
		s<<" <child>"<<m_children[i]<<"</child>\n";
	}
}


QString DDataItemBase::protectXML(const QString & str)
{
	QString ret = str;
	
	ret.replace( "&", "&amp;");
	ret.replace( "<", "&lt;");
	ret.replace( ">", "&gt;");
	ret.replace( "\'", "&apos;" );
	ret.replace( '\"', "&quot;");
	
	return ret;
}

QString DDataItemBase::printXMLTag(const QString & tag, const QString & str, int padding)
{
	QString ret;
	for (int i=0; i<padding; i++)
		ret.append(" ");

	ret.append("<");
	ret.append(tag);
	ret.append(">");

	ret.append(protectXML(str));

	ret.append("</");
	ret.append(tag);
	ret.append(">\n");

	return ret;
}

void DDataItemBase::initColor()
{
	m_fillcolor.setRgb( Settings::itemfillcolor(Type()).rgb() );
	m_outlinecolor.setRgb( Settings::itemoutlinecolor(Type()).rgb() );
	m_textcolor.setRgb( Settings::itemoutlinecolor(Type()).rgb() );
}

