/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qstring.h>
#include <qcolor.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qfont.h>

#include <kdebug.h>

#include "DDataItemBase.h"
#include "DDataItemRoot.h"
#include "DDataItemText.h"
#include "DDataItemPix.h"

#include "DDataControl.h"
#include "DissertParser.h"


DissertParser::DissertParser(DDataControl*d) : QXmlDefaultHandler() 
{
	m_id = -1;
	m_type = DItem::Unknown;
	m_inItem = false;

	m_datacontrol=d;
}

bool DissertParser::startDocument()
{
	return true;
}

bool DissertParser::startElement( const QString&, const QString&localname, const QString& qName, const QXmlAttributes&)
{
	if (qName == "item")
	{
		m_inItem = true;
	}
	return true;
}

bool DissertParser::endElement( const QString&, const QString&, const QString& qName )
{
	if (qName == "id")
	{
		m_id = m_str.toInt();
	}
	else if (qName == "type")
	{
		m_type =TypeFromName(m_str);
		m_datacontrol->createItem(m_type, m_id);
		// kdWarning()<<"adding item of type : "<<m_str<<" or "<<m_type<<" and id "<<m_id<<endl;
		if (!m_datacontrol->isRegistered(m_id))
		{
			kdWarning()<<"BUG : item was not created !"<<endl;
		}
	}
	else if (qName == "xpos")
	{
		DDataItemBase *item = m_datacontrol->dataItem(m_id);
		if (!item)
			kdWarning()<<"BUG : no item at xpos"<<endl;
		item->setXY( m_str.toInt(), item->y());
	}
	else if (qName == "ypos")
	{
		DDataItemBase *item = m_datacontrol->dataItem(m_id);
		if (!item)
			kdWarning()<<"BUG : no item at ypos"<<endl;
		item->setXY( item->x(), m_str.toInt());
	}
	else if (qName == "parent")
	{
		DDataItemBase *item = m_datacontrol->dataItem(m_id);
		if (!item)
			kdWarning()<<"BUG : no item at parent"<<endl;
		item->setParent(m_str.toInt());
	}
	else if (qName == "child")
	{
		DDataItemBase *item = m_datacontrol->dataItem(m_id);
		if (!item)
			kdWarning()<<"BUG : no item at child"<<endl;

		item->addChild(m_str.toInt());
	}
	else if (qName == "defaultfont")
	{
		DDataItemBase *item = m_datacontrol->dataItem(m_id);
		if (!item)
			kdWarning()<<"BUG : no item at font"<<endl;
		item->m_defaultFont.fromString(m_str);
	}
	else if (qName == "fillcolor")
	{
		DDataItemBase *item = m_datacontrol->dataItem(m_id);
		if (!item)
			kdWarning()<<"BUG : no item at fillcolor"<<endl;
		item->m_fillcolor.setNamedColor(m_str);
	}
	else if (qName == "outlinecolor")
	{
		DDataItemBase *item = m_datacontrol->dataItem(m_id);
		if (!item)
			kdWarning()<<"BUG : no item at outlinecolor"<<endl;
		item->m_outlinecolor.setNamedColor(m_str);
	}
	else if (qName == "textcolor")
	{
		DDataItemBase *item = m_datacontrol->dataItem(m_id);
		if (!item)
			kdWarning()<<"BUG : no item at textcolor"<<endl;
		item->m_textcolor.setNamedColor(m_str);
	}

	// now type-specific stuff
	DDataItemBase *item = m_datacontrol->dataItem(m_id);
	switch (m_type)
	{
		case DItem::Root:
			{
				DDataItemRoot* iroot = (DDataItemRoot*) item;
				if (qName == "title")
					iroot->m_title = m_str;
				if (qName == "abstract")
					iroot->m_abstract = m_str;
			}
			break;
		case DItem::Text:
			{
				DDataItemText* itext = (DDataItemText*) item;
				if (qName == "text")
					itext->m_text = m_str;
			}
			break;
		case DItem::Pix:
			{
				DDataItemPix* ipix = (DDataItemPix*) item;
				if (qName == "url")
					ipix->m_url = m_str;
				if (qName == "legend")
					ipix->m_legend = m_str;
			}
			break;
		default:
			break;
	}

	// going to next item
	if (qName == "item")
	{
		m_inItem = false;
		m_type = DItem::Unknown;
		m_id = -1;
	}

	return true;
}

bool DissertParser::characters( const QString& s)
{
	m_str = s;
	return true;
}




