/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdragobject.h>
#include <kprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qdragobject.h>
#include <qstatusbar.h>

#include <kdebug.h>

#include <ktip.h>
#include <kio/netaccess.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kurlrequesterdlg.h>
#include <kconfig.h>

#include <kfiledialog.h>
#include <kactionclasses.h>
#include <kconfigdialog.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kkeydialog.h>

#include <kedittoolbar.h>

#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>

#include "kdissertview.h"
#include "kdissert.h"
#include "settings.h"
#include "prefs.h"

kdissert::kdissert()
	: KMainWindow( 0, "kdissert" ),
	m_view(new kdissertView(this)),
m_printer(0)
{
	// accept dnd
	setAcceptDrops(true);

	// tell the KMainWindow that this is indeed the main widget
	setCentralWidget(m_view);

	// then, setup our actions
	setupActions();

	// and a status bar
	statusBar()->show();

	// apply the saved mainwindow settings, if any, and ask the mainwindow
	// to automatically save settings if changed: window size, toolbar
	// position, icon size, etc.
	setAutoSaveSettings();

	// allow the view to change the statusbar and caption
	connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
			this,   SLOT(changeStatusbar(const QString&)));
	connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
			this,   SLOT(changeCaption(const QString&)));

	showTipOnStart();
}

kdissert::~kdissert()
{
	KConfig *cfg = new KConfig("kdissertrc");
	m_fileOpenRecent->saveEntries(cfg, "RecentFiles");
	delete cfg;
}

void kdissert::setupActions()
{
	KStdAction::open(this, SLOT(fileOpen()), actionCollection());

	m_fileOpenRecent = KStdAction::openRecent(this, SLOT(openURL(const KURL&)), actionCollection());

	KConfig *cfg = new KConfig("kdissertrc");
	m_fileOpenRecent->loadEntries(cfg, "RecentFiles");
	delete cfg;
	
	new KAction(i18n("&Clear"), 0, 
			m_view, SLOT(clearDocument()), actionCollection(),
			"kdissert_clear");
	
	KStdAction::tipOfDay( this, SLOT( showTip() ), actionCollection() 
			)->setWhatsThis(i18n("This shows useful tips on the use of this application."));
	
	new KAction(i18n("&Generate Documents..."), 0,
			m_view, SLOT(generateDocument()), actionCollection(),
			"kdissert_generate");

	new KAction(i18n("&Create Picture..."), 0,
			m_view, SLOT(savePic()), actionCollection(),
			"kdissert_savepic");
	
	KStdAction::save(this, SLOT(fileSave()), actionCollection());
	KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
	KStdAction::quit(kapp, SLOT(quit()), actionCollection());

	m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()), actionCollection());
	m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());
	KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
	KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

	
	KAction* action = KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
	action->setEnabled(false);
	

	KRadioAction * select = new KRadioAction(i18n("&Select item"), "kdissert_point", 0, 
			m_view, SLOT(setPointMode()), actionCollection(),
			"kdissert_point");
	KRadioAction * link = new KRadioAction(i18n("&Link items"), "kdissert_link", 0, 
			m_view, SLOT(setLinkMode()), actionCollection(),
			"kdissert_link");
	KRadioAction * sort = new KRadioAction(i18n("S&ort item"), "kdissert_sort", 0, 
			m_view, SLOT(setSortMode()), actionCollection(),
			"kdissert_sort");

	select->setExclusiveGroup("modes");
	link->setExclusiveGroup("modes");
	sort->setExclusiveGroup("modes");

	select->setChecked(true);

	createGUI();
}

void kdissert::showTip() 
{
	KTipDialog::showTip(this,QString::null,true);
}

void kdissert::showTipOnStart()
{
	//kdWarning()<<"startup tip"<<endl;
        KTipDialog::showTip(this);
}


void kdissert::fileOpen()
{
        // standard filedialog
        KURL url = KFileDialog::getOpenURL(QString::null, i18n("*.kdi|kdissert project (*.kdi)"), this, i18n("Open kdissert project"));
        if (url.isEmpty())
                return;

        m_view->loadFromFile(url);
	m_fileOpenRecent->addURL(url);
	m_url = url;
}

void kdissert::fileSave()
{
        if ( m_url.isEmpty() )
	{
                fileSaveAs();
	}
        else
	{
                m_view->saveToFile(m_url);
		statusBar()->message(i18n("The project was saved"), 5000);
	}
}

void kdissert::fileSaveAs()
{
        // this slot is called whenever the File->Save As menu is selected,
        m_url = KFileDialog::getSaveURL(QString::null, i18n("*.kdi|kdissert project (*.kdi)"), this, i18n("Save kdissert project"));
        if (!m_url.isEmpty() && m_url.isValid())
                fileSave();	
}

void kdissert::optionsShowToolbar()
{
	// this is all very cut and paste code for showing/hiding the
	// toolbar
	if (m_toolbarAction->isChecked())
		toolBar()->show();
	else
		toolBar()->hide();
}

void kdissert::optionsShowStatusbar()
{
	// show/hide the statusbar
	if (m_statusbarAction->isChecked())
		statusBar()->show();
	else
		statusBar()->hide();
}

void kdissert::optionsConfigureKeys()
{
	KKeyDialog::configure(actionCollection());
}


void kdissert::optionsConfigureToolbars()
{
	saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
}

/*void kdissert::newToolbarConfig()
{
	// this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
	// recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
	createGUI();

	applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
}*/

void kdissert::optionsPreferences()
{
	// The preference dialog is derived from prefs-base.ui which is subclassed into Prefs
	//
	// compare the names of the widgets in the .ui file 
	// to the names of the variables in the .kcfg file
	KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self(), KDialogBase::Swallow);
	dialog->addPage(new Prefs(), i18n("General"), "package_settings");
	connect(dialog, SIGNAL(settingsChanged()), m_view, SLOT(settingsChanged()));
	dialog->show();
}

void kdissert::changeStatusbar(const QString& text)
{
	// display the text on the statusbar
	statusBar()->message(text, 2000);
}

void kdissert::changeCaption(const QString& text)
{
	// display the text on the caption
	setCaption(text);
}

void kdissert::dragEnterEvent(QDragEnterEvent *event)
{
    // accept uri drops only
    event->accept(KURLDrag::canDecode(event));
}

void kdissert::dropEvent(QDropEvent *event)
{
    KURL::List urls;

    kdWarning()<<"kdissert::dropEvent"<<endl;
    
    // see if we can decode a URI.. if not, just ignore it
    if (KURLDrag::decode(event, urls) && !urls.isEmpty())
    {
        // okay, we have a URI.. process it
        const KURL &url = urls.first();

        // load in the file
        m_view->loadFromFile(url);
    }
}

void kdissert::openURL( const KURL& url )
{
	if (url.isEmpty()) 
	{
		fileOpen();
		return;
	}
	
	m_view->loadFromFile( url );
	m_fileOpenRecent->addURL(url);
	m_url = url;
}

#include "kdissert.moc"

