/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KDISSERT_H
#define _KDISSERT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapplication.h>
#include <kmainwindow.h>

class kdissertView;
class KPrinter;
class KToggleAction;
class KURL;
class KRecentFilesAction;

/**
 * This class serves as the main window for kdissert.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author ita <tnagy256@yahoo.fr>
 * @version 0.2.1
 */
class kdissert : public KMainWindow
{
	Q_OBJECT
	public:
		kdissert();
		virtual ~kdissert();
		void showTipOnStart();

	public slots:
		void openURL( const KURL& );

	protected:
		virtual void dragEnterEvent(QDragEnterEvent *event);
		virtual void dropEvent(QDropEvent *event);

	private slots:
		void fileOpen();
		void fileSave();
		void fileSaveAs();
		void showTip();

		void optionsShowToolbar();
		void optionsShowStatusbar();
		void optionsConfigureKeys();
		void optionsConfigureToolbars();
		void optionsPreferences();
		//void newToolbarConfig();

		void changeStatusbar(const QString& text);
		void changeCaption(const QString& text);

	private:
		void setupAccel();
		void setupActions();


		kdissertView *m_view;
		KURL m_url;
		QString m_fileName;
		
		KPrinter *m_printer;
		KToggleAction *m_toolbarAction;
		KToggleAction *m_statusbarAction;
		KRecentFilesAction *m_fileOpenRecent;
};

#endif // _KDISSERT_H

