/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kdebug.h>
#include <qcolor.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qstring.h>
#include <qimage.h>

#include <kurlrequester.h>
#include <kicondialog.h>
#include <kurl.h>
#include <kmessagebox.h>
#include <klineedit.h>
#include <klocale.h>
#include <kio/netaccess.h>

#include "ItemPixCfg-content.h"
#include "../gui/settings.h"
#include "../datastruct/DDataItemPix.h"
#include "ItemPixCfg.h"

ItemPixCfg::ItemPixCfg(DDataItemBase* pix, QWidget* parent, const char * name) : 
KDialogBase( parent, name, true, "", KDialogBase::Ok | KDialogBase::Cancel, KDialogBase::Ok, false )
{
	m_pixdata = (DDataItemPix*) pix;
	init();
}

ItemPixCfg::~ItemPixCfg()
{

}

void ItemPixCfg::init()
{
	m_content = new ItemPixCfg_content(this);
	setMainWidget(m_content);

	m_content->iconbutton->setMinimumSize(Settings::pixSize(), Settings::pixSize());
	m_content->iconbutton->setMaximumSize(Settings::pixSize(), Settings::pixSize());
	m_content->iconbutton->setFixedSize(Settings::pixSize(), Settings::pixSize());
	m_content->iconbutton->setPixmap( m_pixdata->m_pixtb );

	m_url = KURL(m_pixdata->m_url);
	m_content->lineedit->setText( m_pixdata->m_legend );

	QString filter;
	m_content->urlbox->setFilter(filter);
	m_content->urlbox->setURL( m_pixdata->m_url );


	connect(m_content->urlbox, SIGNAL(urlSelected(const QString&)), this, SLOT(checkPix(const QString &)) );
	connect(m_content->urlbox, SIGNAL(textChanged(const QString&)), this, SLOT(checkPix(const QString &)) );

	resize(500, 200);
} 

void ItemPixCfg::slotOk()
{
	// kdWarning()<<"ItemPixCfg::slotOk"<<endl;

	// old picture is the same as the new one ..
	if (m_pixdata->m_url == m_url.prettyURL() && m_pixdata->m_legend == m_content->lineedit->text() )
	{
		return;
	}
	
	QString tmpFile;
	if (! KIO::NetAccess::download(m_url, tmpFile, NULL))
	{
		QString text;

		if ( m_url.isLocalFile() )
			text = i18n( "Unable to find the picture given %1 !" );
		else
			text = i18n( "Unable to download the picture given ! "
					"Please check that the address %1 is correct." );

		KMessageBox::sorry( NULL, text.arg( m_url.prettyURL() )  );
		return;
	}

	m_pixdata->loadPix(tmpFile);
	m_pixdata->m_legend = m_content->lineedit->text();
	m_pixdata->m_url = m_url.prettyURL();

	accept();
}

void ItemPixCfg::checkPix(const QString &s)
{
	// kdWarning()<<"ItemPixCfg::checkPix"<<endl;

	QImage img;
	if ( !img.load(s) )
		return;

	m_url = KURL(s);
	QPixmap pix;
	pix.convertFromImage(  img.smoothScale( Settings::pixSize(), Settings::pixSize(), QImage::ScaleMin )  );
	m_content->iconbutton->setPixmap( pix );
}

#include "ItemPixCfg.moc"

