/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmap.h>
#include <qpixmap.h>

#include <pwd.h>
#include <sys/types.h>
#include <unistd.h>

#include <klocale.h>
#include <kdebug.h>
#include <ktempdir.h>
#include <kgenericfactory.h>

#include "../../datastruct/DItem.h"
#include "../../datastruct/DDataControl.h"
#include "../../datastruct/DDataItemBase.h"
#include "../../datastruct/DDataItemRoot.h"
#include "../../datastruct/DDataItemText.h"
#include "../../datastruct/DDataItemPix.h"

#include "kdissasciidoc.h"

K_EXPORT_COMPONENT_FACTORY(libkdissasciidoc, kdissasciidocFactory) 
	QObject *kdissasciidocFactory::createObject(QObject *, const char *, const char *, const QStringList &) { return new kdissasciidoc; }


kdissasciidoc::kdissasciidoc() : DGenerator()
{
	m_data = NULL;
}

kdissasciidoc::~kdissasciidoc()
{

}

QString kdissasciidoc::fullName()
{
	return "kdissasciidoc";
}

QString kdissasciidoc::group()
{
	return "plain text";
}

QString kdissasciidoc::description()
{
	return i18n("This template generates a plain text document (one page without pictures). The pictures will be included in the directory created.");
	
}

QString kdissasciidoc::quickstart()
{
	return i18n("Go into the directory created and view main.txt with a text editor (eg: kate)\n");
}

void kdissasciidoc::generate(const QString &str, DDataControl* data)
{
	// kdWarning()<<"kdissert is generating a document.. "<<str<<endl;
	if (!data)
	{
		// kdWarning()<<"missing data"<<endl;
		return;
	}
	m_data = data;

	QDir dir;
	if (! dir.mkdir(str))
	{
		// kdWarning()<<"could not make dir"<<endl;
		return;
	}

	// makefile
	if (! copyfiles(str) )
	{
		// TODO cleanup
		return;
	}

	// main document
	DDataItemRoot *root = findRoot(data);
	if (root)
	{
		writeMaindoc(root, str);
	}
	else
	{
		//TODO cleanup
	}

	// process the pics
	// kdWarning()<<data->countItems()<<endl;
	QString picdirstr = str + "/pics";
	QDir picdir;
	if (! picdir.mkdir(picdirstr))
	{
		// TODO cleanup
		return;
	}
	
	for (int i=0; i<data->countItems();i++)
	{
		DDataItemBase* item = (DDataItemBase*) data->Item(data->idNum(i));
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;

			// TODO cleanup
			// if ! pix->m_pix.save(... then cleanup
			pix->m_pix.save(picdirstr+"/"+"pic-"+QString::number(data->idNum(i))+".png", "PNG");
		}
	}
}

void kdissasciidoc::writeItem(DDataItemBase* item, QTextStream & s, int level)
{
	// kdWarning()<<"kdissasciidoc::writeItem on level"<<level<<"with item"<<item<<endl;
	if (!item)
	{
		// kdWarning()<<"something is broken in kdissasciidoc::writeItem (1)"<<endl;
		return;
	}

	if (level != 0 && item->Type() == DItem::Root)
	{
		// kdWarning()<<"something is broken in kdissasciidoc::writeItem (2)"<<endl;
		return;
	}

	if (level == 0 && item->Type() != DItem::Root)
	{
		// kdWarning()<<"something is broken in kdissasciidoc::writeItem (3)"<<endl;
		return;
	}

	//	
	if (level == 0)
	{
		//TODO : print the abstract
		//
		DDataItemRoot* root = (DDataItemRoot*) item;
		s<<i18n("Abstract : ")<<root->m_abstract<<"\n\n\n";
	}

	if (level == 1)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;
			s<<"*** "<<text->m_text.left(45)<<"\n\n";
			if (text->m_text.length() > 45)
			{
				s<<"\n\n\n"<<text->m_text<<"\n\n";
			}
		}
	}
	else if (level == 2)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;

			s<<"-> "<<text->m_text.left(50)<<"\n\n";
			if (text->m_text.length() > 50)
			{
				s<<text->m_text<<"\n\n";
			}
		}
	}
	else if (level > 2)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;

			s<<text->m_text<<"\n\n";
		}
	}	

	for (int i = 0; i<item->countChildren(); i++)
	{
		DDataItemBase* nextitem = (DDataItemBase*) m_data->Item(item->childNum(i));
		//// kdWarning()<<"nextitem type : "<<nextitem->Type()<<endl;
		//// kdWarning()<<"nextitem id   : "<<nextitem->Id()<<endl;
		writeItem(nextitem, s, level+1);
	}
}

void kdissasciidoc::writeMaindoc(DDataItemRoot *root, const QString & path)
{
	QFile textfile(path+"/main.txt");
	if (!textfile.open(IO_WriteOnly | IO_Append) )
	{
		kdWarning()<<"could not open ascii file for append :-/"<<endl;
		// TODO : cleanup
		return;
	}

	QTextStream u(&textfile);
	if (encodeUTF8())
	{
		u.setEncoding(QTextStream::UnicodeUTF8);
	}

	//////////////////
	// get the author name
	struct passwd *pw = ::getpwuid(getuid());
	if (!pw)
	{
		// TODO : cleanup
		return;
	}

	u<<i18n("Title : ")<<root->m_title<<"\n";
	u<<"Author : "<<QString::fromLocal8Bit( pw->pw_gecos )<<"\n";

	////////////////////////////////////////////////////
	// now the central part, with text, pictures, etc ..

	writeItem(root, u, 0);

	// central part is finished
	////////////////////////////////////////////////////
	textfile.close();
}

DDataItemRoot* kdissasciidoc::findRoot(DDataControl * data)
{
	if (!data)
		return NULL;

	DDataItemRoot* root = NULL;

	for (int i=0; i<data->countItems();i++)
	{
		DDataItemBase* item = (DDataItemBase*) data->Item(data->idNum(i));
		if (item->Type() == DItem::Root)
		{
			root = (DDataItemRoot*) item;
			break;
		}
	}

	return root;
}

#include "kdissasciidoc.moc"

