/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmap.h>
#include <qpixmap.h>

#include <pwd.h>
#include <sys/types.h>
#include <unistd.h>

#include <klocale.h>
#include <kdebug.h>
#include <ktempdir.h>
#include <kgenericfactory.h>

#include "../../datastruct/DItem.h"
#include "../../datastruct/DDataControl.h"
#include "../../datastruct/DDataItemBase.h"
#include "../../datastruct/DDataItemRoot.h"
#include "../../datastruct/DDataItemText.h"
#include "../../datastruct/DDataItemPix.h"

#include "kdisshtmldoc.h"

K_EXPORT_COMPONENT_FACTORY(libkdisshtmldoc, kdisshtmldocFactory) 
	QObject *kdisshtmldocFactory::createObject(QObject *, const char *, const char *, const QStringList &) { return new kdisshtmldoc; }


kdisshtmldoc::kdisshtmldoc() : DGenerator()
{
	m_data = NULL;
}

kdisshtmldoc::~kdisshtmldoc()
{

}

QString kdisshtmldoc::fullName()
{
	return "kdisshtmldoc";
}

QString kdisshtmldoc::group()
{
	return "html";
}

QString kdisshtmldoc::description()
{
	return i18n("This template generates an html document (one page including pictures) that can be viewed using a web browser (konqueror for example)");
}

QString kdisshtmldoc::quickstart()
{
	return i18n("Go into the directory created and view main.html with a web browser (eg: konqueror)\n");
}

void kdisshtmldoc::generate(const QString &str, DDataControl* data)
{
	// kdWarning()<<"kdissert is generating a document.. "<<str<<endl;
	if (!data)
	{
		// kdWarning()<<"missing data"<<endl;
		return;
	}
	m_data = data;

	QDir dir;
	if (! dir.mkdir(str))
	{
		// kdWarning()<<"could not make dir"<<endl;
		return;
	}

	// makefile
	if (! copyfiles(str) )
	{
		// TODO cleanup
		return;
	}

	// main document
	DDataItemRoot *root = findRoot(data);
	if (root)
	{
		writeMaindoc(root, str);
	}
	else
	{
		//TODO cleanup
	}

	// process the pics
	// kdWarning()<<data->countItems()<<endl;
	QString picdirstr = str + "/pics";
	QDir picdir;
	if (! picdir.mkdir(picdirstr))
	{
		// TODO cleanup
		return;
	}
	
	for (int i=0; i<data->countItems();i++)
	{
		DDataItemBase* item = (DDataItemBase*) data->Item(data->idNum(i));
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;

			// TODO cleanup
			// if ! pix->m_pix.save(... then cleanup
			pix->m_pix.save(picdirstr+"/"+"pic-"+QString::number(data->idNum(i))+".png", "PNG");
		}
	}
}

void kdisshtmldoc::writeItem(DDataItemBase* item, QTextStream & s, int level)
{
	// kdWarning()<<"kdisshtmldoc::writeItem on level"<<level<<"with item"<<item<<endl;
	if (!item)
	{
		// kdWarning()<<"something is broken in kdisshtmldoc::writeItem (1)"<<endl;
		return;
	}

	if (level != 0 && item->Type() == DItem::Root)
	{
		// kdWarning()<<"something is broken in kdisshtmldoc::writeItem (2)"<<endl;
		return;
	}

	if (level == 0 && item->Type() != DItem::Root)
	{
		// kdWarning()<<"something is broken in kdisshtmldoc::writeItem (3)"<<endl;
		return;
	}

	//	
	if (level == 0)
	{
		//TODO : print the abstract
		//
		DDataItemRoot* root = (DDataItemRoot*) item;
		s<<"<p>\n";
		s<<DDataItemBase::protectXML(root->m_abstract)<<"\n";
		s<<"</p>\n";
	}

	if (level == 1)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;
			s<<"<h2>"<<DDataItemBase::protectXML(text->m_text).left(45)<<"</h2>\n\n";
			if (DDataItemBase::protectXML(text->m_text).length() > 45)
			{
				s<<"<p>\n";
				s<<DDataItemBase::protectXML(text->m_text);
				s<<"\n</p>\n";
			}
		}
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;
			//legend && url
			s<<"<img src=\"pics/pic-"<<QString::number(pix->Id())<<
				".png\" alt=\"pic-"<<QString::number(pix->Id())<<
				"\" height=\"500\" width=\"500\" />";

			s<<"<p>\n";
			s<<DDataItemBase::protectXML(pix->m_legend);
			s<<"\n</p>\n";
			
			s<<"<!--"<<DDataItemBase::protectXML(pix->m_url)<<" -->\n";
		}
	}
	else if (level == 2)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;

			s<<"<h3>"<<DDataItemBase::protectXML(text->m_text).left(50)<<"</h3>\n\n";
			if (DDataItemBase::protectXML(text->m_text).length() > 50)
			{
				s<<"<p>\n";
				s<<DDataItemBase::protectXML(text->m_text);
				s<<"\n</p>\n";
			}
		}
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;
			//legend && url
			s<<"<img src=\"pics/pic-"<<QString::number(pix->Id())<<
				".png\" alt=\"pic-"<<QString::number(pix->Id())<<
				"\" height=\"500\" width=\"500\" />";

			s<<"<p>\n";
			s<<DDataItemBase::protectXML(pix->m_legend);
			s<<"\n</p>\n";
			
			s<<"<!--"<<DDataItemBase::protectXML(pix->m_url)<<" -->\n";
		}

	}
	else if (level > 2)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;

			s<<"<p>\n";
			s<<DDataItemBase::protectXML(text->m_text)<<"\n";
			s<<"</p>\n";
		}
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;
			//legend && url
			s<<"<img src=\"pics/pic-"<<QString::number(pix->Id())<<
				".png\" alt=\"pic-"<<QString::number(pix->Id())<<
				"\" height=\"500\" width=\"500\" />";

			s<<"<p>\n";
			s<<DDataItemBase::protectXML(pix->m_legend);
			s<<"\n</p>\n";
			
			s<<"<!--"<<DDataItemBase::protectXML(pix->m_url)<<" -->\n";
		}
	}	

	for (int i = 0; i<item->countChildren(); i++)
	{
		DDataItemBase* nextitem = (DDataItemBase*) m_data->Item(item->childNum(i));
		//// kdWarning()<<"nextitem type : "<<nextitem->Type()<<endl;
		//// kdWarning()<<"nextitem id   : "<<nextitem->Id()<<endl;
		writeItem(nextitem, s, level+1);
	}
}

void kdisshtmldoc::writeMaindoc(DDataItemRoot *root, const QString & path)
{
	QFile textfile(path+"/main.html");
	if (!textfile.open(IO_WriteOnly | IO_Append) )
	{
		kdWarning()<<"could not open html file for append :-/"<<endl;
		// TODO : cleanup
		return;
	}

	QTextStream u(&textfile);
	if (encodeUTF8())
	{
		u.setEncoding(QTextStream::UnicodeUTF8);
	}

	/////////////////
	// html header
	u<<"<html>\n";
	u<<"<head>\n";
	u<<"<title>"<<DDataItemBase::protectXML(root->m_title)<<"</title>\n";	

	if (encodeUTF8())
	{
		u<<"<meta http-equiv=Content-Type content=\"text/html; charset=UTF-8\">\n";
	}
	else
	{
		u<<"<meta http-equiv=Content-Type content=\"text/html; charset=ISO-8859-1\">\n";
	}
	u<<"</head>\n<body>\n\n";

	//////////////////
	// get the author name
	struct passwd *pw = ::getpwuid(getuid());
	if (!pw)
	{
		// TODO : cleanup
		return;
	}

	///////////
	// 

	u<<"<h1>"<<DDataItemBase::protectXML(root->m_title)<<"</h1>\n";
	u<<"<!-- Created by : "<<QString::fromLocal8Bit( pw->pw_gecos )<<" -->\n";

	////////////////////////////////////////////////////
	// now the central part, with text, pictures, etc ..

	writeItem(root, u, 0);

	// central part is finished
	////////////////////////////////////////////////////
	u<<"</body>\n</html>\n";
	textfile.close();
}

DDataItemRoot* kdisshtmldoc::findRoot(DDataControl * data)
{
	if (!data)
		return NULL;

	DDataItemRoot* root = NULL;

	for (int i=0; i<data->countItems();i++)
	{
		DDataItemBase* item = (DDataItemBase*) data->Item(data->idNum(i));
		if (item->Type() == DItem::Root)
		{
			root = (DDataItemRoot*) item;
			break;
		}
	}

	return root;
}

#include "kdisshtmldoc.moc"

