/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmap.h>
#include <qpixmap.h>

#include <pwd.h>
#include <sys/types.h>
#include <unistd.h>

#include <klocale.h>
#include <kdebug.h>
#include <ktempdir.h>
#include <kgenericfactory.h>

#include "../../datastruct/DItem.h"
#include "../../datastruct/DDataControl.h"
#include "../../datastruct/DDataItemBase.h"
#include "../../datastruct/DDataItemRoot.h"
#include "../../datastruct/DDataItemText.h"
#include "../../datastruct/DDataItemPix.h"

#include "kdisspdflatexbook.h"

K_EXPORT_COMPONENT_FACTORY(libkdisspdflatexbook, kdisspdflatexbookFactory) 
	QObject *kdisspdflatexbookFactory::createObject(QObject *, const char *, const char *, const QStringList &) { return new kdisspdflatexbook; }


kdisspdflatexbook::kdisspdflatexbook() : DGenerator()
{
	m_data = NULL;
}

kdisspdflatexbook::~kdisspdflatexbook()
{

}

QString kdisspdflatexbook::fullName()
{
	return "kdisspdflatexbook";
}

QString kdisspdflatexbook::group()
{
	return "latex";
}

QString kdisspdflatexbook::description()
{
	return i18n("This template generates a pdflatex book (it can be transformed into a report easily) with headers, fooders and a table of contents. You will need LaTeX for creating the corresponding pdf document.");
}

QString kdisspdflatexbook::quickstart()
{
	return i18n( "Go into the directory created and run 'make'\n"
			"View the document created by running 'make view'");


}

void kdisspdflatexbook::generate(const QString &str, DDataControl* data)
{
	// kdWarning()<<"kdissert is generating a document.. "<<str<<endl;
	if (!data)
	{
		// kdWarning()<<"missing data"<<endl;
		return;
	}
	m_data = data;

	QDir dir;
	if (! dir.mkdir(str))
	{
		// kdWarning()<<"could not make dir"<<endl;
		return;
	}

	// makefile
	if (! copyfiles(str) )
	{
		// TODO cleanup
		return;
	}

	// main document
	DDataItemRoot *root = findRoot(data);
	if (root)
	{
		writeMaindoc(root, str);
	}
	else
	{
		//TODO cleanup
	}

	// process the pics
	// kdWarning()<<data->countItems()<<endl;
	for (int i=0; i<data->countItems();i++)
	{
		DDataItemBase* item = (DDataItemBase*) data->Item(data->idNum(i));
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;

			// TODO cleanup
			// if ! pix->m_pix.save(... then cleanup
			pix->m_pix.save(str+"/"+"pic-"+QString::number(data->idNum(i))+".png", "PNG");
		}
	}
}

void kdisspdflatexbook::writeItem(DDataItemBase* item, QTextStream & s, int level)
{
	// kdWarning()<<"kdisspdflatexbook::writeItem on level"<<level<<"with item"<<item<<endl;
	if (!item)
	{
		// kdWarning()<<"something is broken in kdisspdflatexbook::writeItem (1)"<<endl;
		return;
	}

	if (level != 0 && item->Type() == DItem::Root)
	{
		// kdWarning()<<"something is broken in kdisspdflatexbook::writeItem (2)"<<endl;
		return;
	}

	if (level == 0 && item->Type() != DItem::Root)
	{
		// kdWarning()<<"something is broken in kdisspdflatexbook::writeItem (3)"<<endl;
		return;
	}

	//	
	if (level == 0)
	{
		//TODO : print the abstract
		//
		
		//DDataItemRoot* root = (DDataItemRoot*) item;
		//s<<root->m_abstract<<"\n\n";
	}

	if (level == 1)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;
			s<<"\\chapter{"<<protectTex(text->m_text.left(45))<<"}\n\n";
			if (text->m_text.length() > 45)
			{
				s<<protectTex(text->m_text)<<"\n\n";
			}
		}
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;
			//legend && url
			s<<"\\begin{figure}[htbp]\n";
			s<<"\\begin{center}\n";
			s<<"\\includegraphics[width=10cm]{pic-"<<QString::number(pix->Id())<<"}\n";
			s<<"\\caption{\\footnotesize{"<<protectTex(pix->m_legend)<<"}}\n";
			s<<"%"<<pix->m_url<<"\n";
			s<<"\\end{center}\n";
			s<<"\\end{figure}\n\n";
		}
	}
	else if (level == 2)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;

			s<<"\\section{"<<protectTex(text->m_text.left(50))<<"}\n\n";
			if (text->m_text.length() > 50)
			{
				s<<protectTex(text->m_text)<<"\n\n";
			}
		}
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;
			//legend && url
			s<<"\\begin{figure}[htbp]\n";
			s<<"\\begin{center}\n";
			s<<"\\includegraphics[width=10cm]{pic-"<<QString::number(pix->Id())<<"}\n";
			s<<"\\caption{\\footnotesize{"<<protectTex(pix->m_legend)<<"}}\n";
			s<<"%"<<pix->m_url<<"\n";
			s<<"\\end{center}\n";
			s<<"\\end{figure}\n\n";
		}

	}
	else if (level == 3)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;
			s<<"  \\subsection{"<<protectTex(text->m_text.left(55))<<"}\n\n";
			if (text->m_text.length() > 55)
			{
				s<<protectTex(text->m_text)<<"\n\n";
			}
		}
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;
			//legend && url
			s<<"\\begin{figure}[htbp]\n";
			s<<"\\begin{center}\n";
			s<<"\\includegraphics[width=10cm]{pic-"<<QString::number(pix->Id())<<"}\n";
			s<<"\\caption{\\footnotesize{"<<protectTex(pix->m_legend)<<"}}\n";
			s<<"%"<<pix->m_url<<"\n";
			s<<"\\end{center}\n";
			s<<"\\end{figure}\n\n";
		}
	}
	else if (level == 4)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;

			s<<"    \\subsubsection{"<<protectTex(text->m_text.left(60))<<"}\n\n";
			if (text->m_text.length() > 60)
			{
				s<<protectTex(text->m_text)<<"\n\n";
			}
		}
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;
			//legend && url
			s<<"\\begin{figure}[htbp]\n";
			s<<"\\begin{center}\n";
			s<<"\\includegraphics[width=10cm]{pic-"<<QString::number(pix->Id())<<"}\n";
			s<<"\\caption{\\footnotesize{"<<protectTex(pix->m_legend)<<"}}\n";
			s<<"%"<<pix->m_url<<"\n";
			s<<"\\end{center}\n";
			s<<"\\end{figure}\n\n";
		}
	}
	else if (level > 4)
	{
		if (item->Type() == DItem::Text)
		{
			DDataItemText* text = (DDataItemText*) item;
			s<<protectTex(text->m_text)<<"\n\n";
		}
		if (item->Type() == DItem::Pix)
		{
			DDataItemPix* pix = (DDataItemPix*) item;
			//legend && url
			s<<"\\begin{figure}[htbp]\n";
			s<<"\\begin{center}\n";
			s<<"\\includegraphics[width=10cm]{pic-"<<QString::number(pix->Id())<<"}\n";
			s<<"\\caption{\\footnotesize{"<<protectTex(pix->m_legend)<<"}}\n";
			s<<"%"<<pix->m_url<<"\n";
			s<<"\\end{center}\n";
			s<<"\\end{figure}\n\n";
		}
	}	

	for (int i = 0; i<item->countChildren(); i++)
	{
		DDataItemBase* nextitem = (DDataItemBase*) m_data->Item(item->childNum(i));
		//// kdWarning()<<"nextitem type : "<<nextitem->Type()<<endl;
		//// kdWarning()<<"nextitem id   : "<<nextitem->Id()<<endl;
		writeItem(nextitem, s, level+1);
	}
}

void kdisspdflatexbook::writeMaindoc(DDataItemRoot *root, const QString & path)
{
	QFile textfile(path+"/main.tex");
	if (!textfile.open(IO_WriteOnly | IO_Append) )
	{
		kdWarning()<<"coulnd not open latex file for append :-/"<<endl;
		// TODO : cleanup
		return;
	}

	QTextStream u(&textfile);
	if (encodeUTF8())
	{
		u.setEncoding(QTextStream::UnicodeUTF8);
		u<<"\\usepackage[utf8]{inputenc}\n\n\n";
	}

	//////////////////
	// get the author name
	struct passwd *pw = ::getpwuid(getuid());
	if (!pw)
	{
		// TODO : cleanup
		return;
	}

	///////////
	// print the headers and footers
	u<<"%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n";
	u<<"%document headers and footers\n";
	u<<"\\lhead{}\n";
	u<<"\\chead{}\n";
	u<<"\\rhead{}\n";
	u<<"\\lfoot{"<<QString::fromLocal8Bit( pw->pw_gecos )<<"}\n";
	u<<"\\cfoot{\\thepage/\\pageref{LastPage}}\n";
	u<<"\\rfoot{ \\today } % -> \\rfoot{\\number\\month/\\number\\day/\\number\\year} \n";
	u<<"\\pagestyle{fancyplain}\n";
	u<<"\n\n\n";

	///////////
	// make a nice title for the first page

	u<<"\\title{"<<protectTex(root->m_title)<<"}\n";
	u<<"\\date{}\n";
	u<<"\\author{"<<QString::fromLocal8Bit( pw->pw_gecos )<<"}\n";

	///////////
	// standard headers
	u<<"\n";
	u<<"\\begin{document}\n";
	u<<"\\maketitle\n";
	u<<"\\tableofcontents\n";

	////////////////////////////////////////////////////
	// now the central part, with text, pictures, etc ..

	writeItem(root, u, 0);

	// central part is finished
	////////////////////////////////////////////////////
	u<<"\\end{document}\n\n";
	textfile.close();
}

DDataItemRoot* kdisspdflatexbook::findRoot(DDataControl * data)
{
	if (!data)
		return NULL;

	DDataItemRoot* root = NULL;

	for (int i=0; i<data->countItems();i++)
	{
		DDataItemBase* item = (DDataItemBase*) data->Item(data->idNum(i));
		if (item->Type() == DItem::Root)
		{
			root = (DDataItemRoot*) item;
			break;
		}
	}

	return root;
}

#include "kdisspdflatexbook.moc"

